<?php

/**
 * @file
 * Enables modules and site configuration for a DruStack installation.
 */

/**
 * Implements hook_init.
 */
function drustack_init() {
  $cache = cache_get("drustack_info");
  if (isset($cache->data)) {
    $data = $cache->data;
  }
  else {
    $info = drupal_parse_info_file(dirname(__file__) . '/drustack.info');
    $version = array_key_exists('version', $info) ? $info['version'] : '7.x-16.x-dev';
    $data = array("profile" => "drustack", "profile_version" => $version);
    cache_set("drustack_info", $data);
  }
  drupal_add_js($data, 'setting');
}

/**
 * Implement hook_install_tasks().
 */
function drustack_install_tasks($install_state) {
  return array(
    'drustack_batch' => array(
      'type' => 'batch',
    ),
  );
}

/**
 * Installation step callback.
 */
function drustack_batch(&$install_state) {
  $batch['title'] = st('Installing @drupal', array('@drupal' => drupal_install_profile_distribution_name()));
  $batch['operations'][] = array('_drustack_configure_paths', array());
  $batch['operations'][] = array('_drustack_configure_directories', array());
  $batch['operations'][] = array('_drustack_configure_files', array());
  $batch['operations'][] = array('_drustack_configure_blocks', array());
  $batch['operations'][] = array('_drustack_configure_variables', array());
  $batch['operations'][] = array('_drustack_configure_cleanup', array());
  $batch['error_message'] = st('The installation has encountered an error.');
  return $batch;
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Allows the profile to alter the site configuration form.
 */
function drustack_form_install_configure_form_alter(&$form, $form_state) {
  // Site information form.
  $form['site_information']['#collapsible'] = TRUE;
  $form['site_information']['#collapsed'] = TRUE;
  $form['site_information']['#weight'] = -20;
  $form['site_information']['site_name']['#default_value'] = $_SERVER['SERVER_NAME'];
  $form['site_information']['site_mail']['#default_value'] = 'root@localhost.localdomain';

  // Administrator account form.
  $form['admin_account']['#collapsible'] = TRUE;
  $form['admin_account']['#collapsed'] = TRUE;
  $form['admin_account']['#weight'] = -19;
  $form['admin_account']['account']['name']['#default_value'] = 'root';
  $form['admin_account']['account']['name']['#disabled'] = TRUE;
  $form['admin_account']['account']['mail']['#default_value'] = 'root@localhost.localdomain';
  $form['admin_account']['account']['mail']['#disabled'] = TRUE;
  $form['admin_account']['account']['pass'] = array(
    '#type' => 'value',
    '#value' => user_password(),
  );

  // Power user account form.
  $form['user_account'] = array(
    '#type' => 'fieldset',
    '#title' => st('Site power user account'),
    '#collapsible' => FALSE,
  );

  $form['user_account']['#weight'] = -18;
  $form['user_account']['user_account']['#tree'] = TRUE;
  $form['user_account']['user_account']['name'] = array(
    '#title' => st('Username'),
    '#type' => 'textfield',
    '#default_value' => 'admin',
    '#maxlength' => USERNAME_MAX_LENGTH,
    '#description' => st('Spaces are allowed; punctuation is not allowed except for periods, hyphens, and underscores.'),
    '#required' => TRUE,
    '#weight' => -10,
    '#attributes' => array('class' => array('username')),
  );

  $form['user_account']['user_account']['mail'] = array(
    '#title' => st('E-mail address'),
    '#type' => 'textfield',
    '#default_value' => 'admin@localhost.localdomain',
    '#maxlength' => EMAIL_MAX_LENGTH,
    '#required' => TRUE,
    '#weight' => -5,
  );
  $form['user_account']['user_account']['pass'] = array(
    '#type' => 'password_confirm',
    '#required' => TRUE,
    '#size' => 25,
    '#weight' => 0,
  );

  // Just alter the weight
  $form['server_settings']['#weight'] = -17;
  $form['update_notifications']['#weight'] = -16;
  $form['actions']['#weight'] = -15;

  // Add our own validation
  $form['#validate'][] = '_drustack_form_install_configure_form_validate';

  // Add our own submit handler
  $form['#submit'][] = '_drustack_form_install_configure_form_submit';
}

/**
 * Validate Power user account
 */
function _drustack_form_install_configure_form_validate($form, &$form_state) {
  // Check admin account name.
  if ($error = user_validate_name($form_state['values']['user_account']['name'])) {
    form_error($form['user_account']['user_account']['name'], $error);
  }
  if ($form_state['values']['user_account']['name'] == $form_state['values']['account']['name']) {
    form_error($form['user_account']['user_account']['name'], t('The admin name is not valid.'));
  }

  // Check admin account e-mail address.
  if ($error = user_validate_mail($form_state['values']['user_account']['mail'])) {
    form_error($form['user_account']['user_account']['mail'], $error);
  }
  if ($form_state['values']['user_account']['mail'] == $form_state['values']['account']['mail']) {
    form_error($form['user_account']['user_account']['mail'], t('The admin e-email address is not valid.'));
  }
}

/**
 * Create Power user account and change root password
 */
function _drustack_form_install_configure_form_submit($form, &$form_state) {
  // Add the user and associate role here.
  $power_user_role = user_role_load_by_name('power user');
  if (!$power_user_role) {
    $power_user_role = new stdClass();
    $power_user_role->name = 'power user';
    user_role_save($power_user_role);
    $power_user_role = user_role_load_by_name('power user');
  }
  $user_data = array(
    'mail' => $form_state['values']['user_account']['mail'],
    'name' => $form_state['values']['user_account']['name'],
    'pass' => $form_state['values']['user_account']['pass'],
    'init' => $form_state['values']['user_account']['mail'],
    'status' => 1,
    'roles' => array(
      $power_user_role->rid => $power_user_role->name,
    ),
  );
  $account = user_save(new StdClass(), $user_data);

  // We keep power user and administrator account password in sync by default.
  $root_user = user_load(1);
  $user_data = array(
    'pass' => $form_state['values']['user_account']['pass'],
  );
  user_save($root_user, $user_data);
}

/**
 * Configure files directory as legacy 5.x mode for multi-site installation,
 * also 7.x private file and some other required paths.
 */
function _drustack_configure_paths() {
  // Use legacy 5.x default file_directory_path for multi-site installation.
  $file_public_path = 'files';
  variable_set('file_public_path', $file_public_path);
  file_prepare_directory($file_public_path, FILE_CREATE_DIRECTORY);

  // Also create 7.x default file_private_path for multi-site installation.
  $file_private_path = 'files/private';
  variable_set('file_private_path', $file_private_path);
  file_prepare_directory($file_private_path, FILE_CREATE_DIRECTORY);

  // And update the default download method as public.
  file_get_stream_wrappers(STREAM_WRAPPERS_WRITE_VISIBLE);
  variable_set('file_default_scheme', 'public');
}

/**
 * Create necessary directories.
 */
function _drustack_configure_directories() {
  $dirs = array('styles', 'ctools', 'ctools/css', 'pictures', 'imagecache', 'css', 'js');
  foreach ($dirs as $dir) {
    $dir = 'public://' . $dir;
    file_prepare_directory($dir, FILE_CREATE_DIRECTORY);
  }
}

/**
 * Copy necessary files.
 */
function _drustack_configure_files() {
  $files = array('pictures/user.png');
  foreach ($files as $file) {
    $source = 'profiles/drustack/files/' . $file;
    $destination = 'public://' . $file;
    file_unmanaged_copy($source, $destination, FILE_EXISTS_REPLACE);
  }
}

/**
 * Configure devel blocks for switch user and execute php.
 */
function _drustack_configure_blocks() {
  $default_theme = variable_get('theme_default', 'bartik');

  // Enable some blocks.
  db_update('block')
    ->fields(array(
      'status' => 1,
      'region' => 'footer',
    ))
    ->condition('module', 'devel')
    ->condition('delta', 'execute_php')
    ->condition('theme', $default_theme)
    ->execute();
}

/**
 * Configure extra variables.
 */
function _drustack_configure_variables() {
  variable_set('block_cache', 0);
  variable_set('cache', 0);
  variable_set('cache_lifetime', 0);
  variable_set('dblog_row_limit', 1000);
  variable_set('error_level', 2);
  variable_set('googleanalytics_account', 'UA-0000000-00');
  variable_set('page_cache_maximum_age', 0);
  variable_set('preprocess_css', 0);
  variable_set('preprocess_js', 0);
  variable_set('seo_checker_article', 1);
  variable_set('seo_checker_blog', 1);
  variable_set('seo_checker_page', 1);
  variable_set('update_check_disabled', 1);
  variable_set('user_picture_default', 'public://pictures/user.png');
  variable_set('user_picture_file_size', 1024);
}

/**
 * Various actions needed to clean up after the installation.
 */
function _drustack_configure_cleanup() {
  // Flush all existing path aliases.
  db_delete('url_alias');

  // Rebuild node access database.
  node_access_rebuild();

  // Rebuild node types.
  node_types_rebuild();

  // Rebuild the menu.
  menu_rebuild();

  // Clear drupal message queue for non-warning/errors.
  drupal_get_messages('status', TRUE);

  // Clear out caches
  drupal_flush_all_caches();

  // Clear out JS and CSS caches.
  drupal_clear_css_cache();
  drupal_clear_js_cache();

  // Revert all features.
  features_revert();

  // Enable git_deploy if clone from GIT.
  $info = drupal_parse_info_file(dirname(__file__) . '/drustack.info');
  $version = array_key_exists('version', $info) ? $info['version'] : '7.x-16.x-dev';
  if ($version == '7.x-16.x-dev') {
    module_enable(array('git_deploy'), FALSE);
  }
}
